@@ -87,8 +87,8 @@ c_size_t_p = POINTER(c_size_t)
 # llama.h bindings
 
 GGML_USE_CUBLAS = hasattr(_lib, "ggml_init_cublas")
-GGML_CUDA_MAX_DEVICES = ctypes.c_int(16)
-LLAMA_MAX_DEVICES = GGML_CUDA_MAX_DEVICES if GGML_USE_CUBLAS else ctypes.c_int(1)
+GGML_CUDA_MAX_DEVICES = 16
+LLAMA_MAX_DEVICES = GGML_CUDA_MAX_DEVICES if GGML_USE_CUBLAS else 1
 
 # define LLAMA_DEFAULT_SEED 0xFFFFFFFF
 LLAMA_DEFAULT_SEED = ctypes.c_int(0xFFFFFFFF)
@@ -335,13 +335,13 @@ llama_grammar_p = c_void_p
 #     // LLAMA_GRETYPE_CHAR_RNG_UPPER to add an alternate char to match ([ab], [a-zA])
 #     LLAMA_GRETYPE_CHAR_ALT       = 6,
 # };
-LLAMA_GRETYPE_END = c_int(0)
-LLAMA_GRETYPE_ALT = c_int(1)
-LLAMA_GRETYPE_RULE_REF = c_int(2)
-LLAMA_GRETYPE_CHAR = c_int(3)
-LLAMA_GRETYPE_CHAR_NOT = c_int(4)
-LLAMA_GRETYPE_CHAR_RNG_UPPER = c_int(5)
-LLAMA_GRETYPE_CHAR_ALT = c_int(6)
+LLAMA_GRETYPE_END = 0
+LLAMA_GRETYPE_ALT = 1
+LLAMA_GRETYPE_RULE_REF = 2
+LLAMA_GRETYPE_CHAR = 3
+LLAMA_GRETYPE_CHAR_NOT = 4
+LLAMA_GRETYPE_CHAR_RNG_UPPER = 5
+LLAMA_GRETYPE_CHAR_ALT = 6
 
 
 # typedef struct llama_grammar_element {
@@ -407,7 +407,7 @@ _lib.llama_model_quantize_default_params.restype = llama_model_quantize_params
 # // If numa is true, use NUMA optimizations
 # // Call once at the start of the program
 # LLAMA_API void llama_backend_init(bool numa);
-def llama_backend_init(numa: c_bool):
+def llama_backend_init(numa: Union[c_bool, bool]):
     return _lib.llama_backend_init(numa)
 
 
@@ -651,9 +651,9 @@ _lib.llama_model_quantize.restype = c_int
 #                          int   n_threads);
 def llama_apply_lora_from_file(
     ctx: llama_context_p,
-    path_lora: c_char_p,
-    path_base_model: c_char_p,
-    n_threads: c_int,
+    path_lora: Union[c_char_p, bytes],
+    path_base_model: Union[c_char_p, bytes],
+    n_threads: Union[c_int, int],
 ) -> int:
     return _lib.llama_apply_lora_from_file(ctx, path_lora, path_base_model, n_threads)
 
@@ -671,7 +671,7 @@ def llama_model_apply_lora_from_file(
     model: llama_model_p,
     path_lora: Union[c_char_p, bytes],
     path_base_model: Union[c_char_p, bytes],
-    n_threads: c_int,
+    n_threads: Union[c_int, int],
 ) -> int:
     return _lib.llama_model_apply_lora_from_file(
         model, path_lora, path_base_model, n_threads
@@ -751,7 +751,7 @@ def llama_load_session_file(
     ctx: llama_context_p,
     path_session: bytes,
     tokens_out,  # type: Array[llama_token]
-    n_token_capacity: c_size_t,
+    n_token_capacity: Union[c_size_t, int],
     n_token_count_out,  # type: _Pointer[c_size_t]
 ) -> int:
     return _lib.llama_load_session_file(
@@ -774,7 +774,7 @@ def llama_save_session_file(
     ctx: llama_context_p,
     path_session: bytes,
     tokens,  # type: Array[llama_token]
-    n_token_count: c_size_t,
+    n_token_count: Union[c_size_t, int],
 ) -> int:
     return _lib.llama_save_session_file(ctx, path_session, tokens, n_token_count)
 
@@ -801,9 +801,9 @@ _lib.llama_save_session_file.restype = c_size_t
 def llama_eval(
     ctx: llama_context_p,
     tokens,  # type: Array[llama_token]
-    n_tokens: c_int,
-    n_past: c_int,
-    n_threads: c_int,
+    n_tokens: Union[c_int, int],
+    n_past: Union[c_int, int],
+    n_threads: Union[c_int, int],
 ) -> int:
     return _lib.llama_eval(ctx, tokens, n_tokens, n_past, n_threads)
 
@@ -822,9 +822,9 @@ _lib.llama_eval.restype = c_int
 def llama_eval_embd(
     ctx: llama_context_p,
     embd,  # type: Array[c_float]
-    n_tokens: c_int,
-    n_past: c_int,
-    n_threads: c_int,
+    n_tokens: Union[c_int, int],
+    n_past: Union[c_int, int],
+    n_threads: Union[c_int, int],
 ) -> int:
     return _lib.llama_eval_embd(ctx, embd, n_tokens, n_past, n_threads)
 
@@ -1042,8 +1042,8 @@ _lib.llama_token_to_piece_with_model.restype = c_int
 #                                 size_t    start_rule_index);
 def llama_grammar_init(
     rules,  # type: Array[llama_grammar_element_p] # type: ignore
-    n_rules: c_size_t,
-    start_rule_index: c_size_t,
+    n_rules: Union[c_size_t, int],
+    start_rule_index: Union[c_size_t, int],
 ) -> llama_grammar_p:
     return _lib.llama_grammar_init(rules, n_rules, start_rule_index)
 
@@ -1084,8 +1084,8 @@ def llama_sample_repetition_penalty(
     ctx: llama_context_p,
     candidates,  # type: _Pointer[llama_token_data_array]
     last_tokens_data,  # type: Array[llama_token]
-    last_tokens_size: c_int,
-    penalty: c_float,
+    last_tokens_size: Union[c_int, int],
+    penalty: Union[c_float, float],
 ):
     return _lib.llama_sample_repetition_penalty(
         ctx, candidates, last_tokens_data, last_tokens_size, penalty
@@ -1108,9 +1108,9 @@ def llama_sample_frequency_and_presence_penalties(
     ctx: llama_context_p,
     candidates,  # type: _Pointer[llama_token_data_array]
     last_tokens_data,  # type: Array[llama_token]
-    last_tokens_size: c_int,
-    alpha_frequency: c_float,
-    alpha_presence: c_float,
+    last_tokens_size: Union[c_int, int],
+    alpha_frequency: Union[c_float, float],
+    alpha_presence: Union[c_float, float],
 ):
     return _lib.llama_sample_frequency_and_presence_penalties(
         ctx,
@@ -1146,7 +1146,7 @@ def llama_sample_classifier_free_guidance(
     ctx: llama_context_p,
     candidates,  # type: _Pointer[llama_token_data_array]
     guidance_ctx: llama_context_p,
-    scale: c_float,
+    scale: Union[c_float, float],
 ):
     return _lib.llama_sample_classifier_free_guidance(
         ctx, candidates, guidance_ctx, scale
@@ -1182,8 +1182,8 @@ _lib.llama_sample_softmax.restype = None
 def llama_sample_top_k(
     ctx: llama_context_p,
     candidates,  # type: _Pointer[llama_token_data_array]
-    k: c_int,
-    min_keep: c_size_t,
+    k: Union[c_int, int],
+    min_keep: Union[c_size_t, int],
 ):
     return _lib.llama_sample_top_k(ctx, candidates, k, min_keep)
 
@@ -1202,8 +1202,8 @@ _lib.llama_sample_top_k.restype = None
 def llama_sample_top_p(
     ctx: llama_context_p,
     candidates,  # type: _Pointer[llama_token_data_array]
-    p: c_float,
-    min_keep: c_size_t,
+    p: Union[c_float, float],
+    min_keep: Union[c_size_t, int],
 ):
     return _lib.llama_sample_top_p(ctx, candidates, p, min_keep)
 
@@ -1222,8 +1222,8 @@ _lib.llama_sample_top_p.restype = None
 def llama_sample_tail_free(
     ctx: llama_context_p,
     candidates,  # type: _Pointer[llama_token_data_array]
-    z: c_float,
-    min_keep: c_size_t,
+    z: Union[c_float, float],
+    min_keep: Union[c_size_t, int],
 ):
     return _lib.llama_sample_tail_free(ctx, candidates, z, min_keep)
 
@@ -1242,8 +1242,8 @@ _lib.llama_sample_tail_free.restype = None
 def llama_sample_typical(
     ctx: llama_context_p,
     candidates,  # type: _Pointer[llama_token_data_array]
-    p: c_float,
-    min_keep: c_size_t,
+    p: Union[c_float, float],
+    min_keep: Union[c_size_t, int],
 ):
     return _lib.llama_sample_typical(ctx, candidates, p, min_keep)
 
@@ -1261,7 +1261,7 @@ _lib.llama_sample_typical.restype = None
 def llama_sample_temperature(
     ctx: llama_context_p,
     candidates,  # type: _Pointer[llama_token_data_array]
-    temp: c_float,
+    temp: Union[c_float, float],
 ):
     return _lib.llama_sample_temperature(ctx, candidates, temp)
 
@@ -1301,9 +1301,9 @@ _lib.llama_sample_grammar.restype = None
 def llama_sample_token_mirostat(
     ctx: llama_context_p,
     candidates,  # type: _Pointer[llama_token_data_array]
-    tau: c_float,
-    eta: c_float,
-    m: c_int,
+    tau: Union[c_float, float],
+    eta: Union[c_float, float],
+    m: Union[c_int, int],
     mu,  # type: _Pointer[c_float]
 ) -> int:
     return _lib.llama_sample_token_mirostat(ctx, candidates, tau, eta, m, mu)
@@ -1329,8 +1329,8 @@ _lib.llama_sample_token_mirostat.restype = llama_token
 def llama_sample_token_mirostat_v2(
     ctx: llama_context_p,
     candidates,  # type: _Pointer[llama_token_data_array]
-    tau: c_float,
-    eta: c_float,
+    tau: Union[c_float, float],
+    eta: Union[c_float, float],
     mu,  # type: _Pointer[c_float]
 ) -> int:
     return _lib.llama_sample_token_mirostat_v2(ctx, candidates, tau, eta, mu)
@@ -1531,5 +1531,5 @@ _lib.llama_dump_timing_info_yaml.restype = None
 _llama_initialized = False
 
 if not _llama_initialized:
-    llama_backend_init(c_bool(False))
+    llama_backend_init(False)
     _llama_initialized = True
